==> pgljupyter-0.1.19.tar.gz: package/lib/widgets/decoder.js <==
import pgljs from '../pgljs/dist/index.js';
let MAX_WORKER = 10;
const workers = new Map();
const getWorker = () => {
    const avg = Array.from(workers.values()).reduce((s, v) => s + v.length / workers.size, 0);
    if ((workers.size === 0 || avg >= 1) && workers.size < MAX_WORKER) {
        const worker = pgljs();
        workers.set(worker, []);
        worker.onerror = function (evt) {
            console.log(evt);
        };
        worker.onmessage = function (evt) {
            if (evt.data && 'initialized' in evt.data) {
                if (evt.data.initialized) {
                    // console.log('initialized');
                    workers.get(this).forEach(task => this.postMessage(task.data));
                    this.initialized = true;
                }
                else {
                    // console.log('terminate on error');
                    workers.get(this).forEach(task => task.reject({ err: 'initialization failed', userData: task.userData }));
                    this.terminate();
                    workers.delete(this);
                }
            }
            else {
                if (workers.get(this).length) {
                    const { resolve, reject, userData } = workers.get(this).shift();
                    if (evt.data.error) {
                        reject({ error: evt.data.error, userData });
                    }
                    else {
                        resolve({ results: evt.data, userData });
                    }
                }
            }
            if (workers.get(this).length === 0) {
                setTimeout((self => (() => {
                    if (workers.get(self) && workers.get(self).length === 0) {
                        workers.delete(self);
                        // console.log('terminated');
                        self.terminate();
                    }
                }))(this), 60000);
            }
        };
        // console.log('workers', Array.from(workers.values()).map(w => w.length));
        return worker;
    }
    else {
        let worker = workers.keys().next().value;
        for (const key of workers.keys()) {
            if (!workers.get(key).length) {
                worker = key;
                break;
            }
            if (workers.get(worker).length > workers.get(key).length) {
                worker = key;
            }
        }
        // console.log('workers', Array.from(workers.values()).map(w => w.length));
        return worker;
    }
};
class Decoder {
    decode(task, taskId = '') {
        const worker = getWorker();
        return new Promise((resolve, reject) => {
            workers.get(worker).push(Object.assign({ taskId: taskId, resolve, reject }, task));
            if (worker.initialized) {
                worker.postMessage(task.data);
            }
        });
    }
    abort(taskId) {
        let tasks = [];
        for (const worker of workers.keys()) {
            tasks = [...tasks, ...workers.get(worker).filter(task => task.taskId === taskId)];
            workers.set(worker, workers.get(worker).filter(task => task.taskId !== taskId));
        }
        // reject in order
        tasks.sort((a, b) => a.userData.no - b.userData.no)
            .forEach(task => task.reject({ abort: true, userData: task.userData }));
    }
}
;
export default new Decoder();
//# sourceMappingURL=decoder.js.map

==> pgljupyter-1.0.0.tar.gz: package/lib/widgets/decoder.js <==
import pgljs from '../pgljs/dist/index.js';
let MAX_WORKER = 10;
const workers = new Map();
const getWorker = () => {
    const avg = Array.from(workers.values()).reduce((s, v) => s + v.length / workers.size, 0);
    if ((workers.size === 0 || avg >= 1) && workers.size < MAX_WORKER) {
        const worker = pgljs();
        workers.set(worker, []);
        worker.onerror = function (evt) {
            console.log(evt);
        };
        worker.onmessage = function (evt) {
            if (evt.data && 'initialized' in evt.data) {
                if (evt.data.initialized) {
                    // console.log('initialized');
                    workers.get(this).forEach(task => this.postMessage(task.data));
                    this.initialized = true;
                }
                else {
                    // console.log('terminate on error');
                    workers.get(this).forEach(task => task.reject({ err: 'initialization failed', userData: task.userData }));
                    this.terminate();
                    workers.delete(this);
                }
            }
            else {
                if (workers.get(this).length) {
                    const { resolve, reject, userData } = workers.get(this).shift();
                    if (evt.data.error) {
                        reject({ error: evt.data.error, userData });
                    }
                    else {
                        resolve(Object.assign(Object.assign({}, evt.data), { userData }));
                    }
                }
            }
            if (workers.get(this).length === 0) {
                setTimeout((self => (() => {
                    if (workers.get(self) && workers.get(self).length === 0) {
                        workers.delete(self);
                        // console.log('terminated');
                        self.terminate();
                    }
                }))(this), 60000);
            }
        };
        // console.log('workers', Array.from(workers.values()).map(w => w.length));
        return worker;
    }
    else {
        let worker = workers.keys().next().value;
        for (const key of workers.keys()) {
            if (!workers.get(key).length) {
                worker = key;
                break;
            }
            if (workers.get(worker).length > workers.get(key).length) {
                worker = key;
            }
        }
        // console.log('workers', Array.from(workers.values()).map(w => w.length));
        return worker;
    }
};
class Decoder {
    decode(task, taskId = '') {
        const worker = getWorker();
        return new Promise((resolve, reject) => {
            workers.get(worker).push(Object.assign({ taskId: taskId, resolve, reject }, task));
            if (worker.initialized) {
                worker.postMessage(task.data);
            }
        });
    }
    abort(taskId) {
        let tasks = [];
        for (const worker of workers.keys()) {
            tasks = [...tasks, ...workers.get(worker).filter(task => task.taskId === taskId)];
            workers.set(worker, workers.get(worker).filter(task => task.taskId !== taskId));
        }
        // reject in order
        tasks.sort((a, b) => a.userData.no - b.userData.no)
            .forEach(task => task.reject({ abort: true, userData: task.userData }));
    }
}
;
export default new Decoder();
//# sourceMappingURL=decoder.js.map
